# AC3D
# Copyright (c) 2016, Inivis Limited. All rights reserved.

# @version: $Id: ac3dttfont.tcl,v 1.8 2019/02/13 17:27:19 ac Exp $
# @author:  Inivis, Rune Madsen <Rune@DataIT.net>

# TTFont TCL code for a dialog that gets lets the user select a true type (or other) font,
# enter text to be rendered, as well as various font parameters.
#
# Note - the standard tk_GetOpenFile dialog doesn't work for selecting fonts in Windows
# because Microsoft in all their wisdom decided nobody should be able to open files in
# the system font directory - it's not just a tcl/tk thing.



proc ttfont_get_fontdir { current } {
	set filename [tk_chooseDirectory -title "Select font folder" \
		    -initialdir $current -mustexist 1 -parent . ]
	return $filename
}


proc ttfont_fill_font_list { w fdir } {

	global fontname
	global fontdir
	global prefs_ttfont_path

	regsub "\\\\" $fdir "/" fdir
	regsub "//+" $fdir "/" fdir
	set fontdir $fdir
	set prefs_ttfont_path $fdir

	set files [glob -nocomplain "$fdir/*.ttf"]
	set list [lsort -dictionary -increasing $files]

	$w delete 0 end

	foreach f $list {
		regsub ".*/" $f "" f
		$w insert end $f
	}
}



proc ttfont_launch_create {} {

	global fontdir
	global fontname
	global fonttext
	global objecttype
	global pointsize
	global extrudedepth
	global beveldepth
	global bevelangle
	global fontdetail

	if { $fontname=="" } {
		tk_messageBox -type ok -title "Text creation error" -message "Please select a font."
		return
	}

	if { $fonttext=="" } {
		tk_messageBox -type ok -title "Text creation error" -message "Please enter text, then click the \"create\" button."
		return
	}

	ac3d ttfont_create_text $fontdir/$fontname $fonttext $objecttype $pointsize $extrudedepth $beveldepth $bevelangle $fontdetail
}


ac3d add_pref window_geom_ttfont ""

set fonttext ""

proc ttfont_create_win { parent } {

	global env
	global tcl_platform
	global prefs_ttfont_path

	global ttinitialize
	global fontlistwidget
	global fontdir

	global fonttext
	global objecttype
	global pointsize
	global extrudedepth
	global beveldepth
	global bevelangle
	global fontdetail
	global prefs_window_geom_ttfont
	
	set fontdir $prefs_ttfont_path

	if { $fontdir == "" } {

		if { $tcl_platform(platform) == "windows" } {

			if [ info exists env(windir) ] {
				set fontdir $env(windir)/FONTS
			} elseif [ info exists env(systemroot) ] {
				set fontdir $env(systemroot)/FONTS
			} else {
				set fontdir "c:/windows/fonts"
			}

		} elseif { $tcl_platform(platform) == "unix" } {
			set fontdir "/usr/share/fonts/truetype"
		} else {
			set fontdir ""
		}
	}
	set prefs_ttfont_path $fontdir

	if [info exists ttinitialize] {
		set ttinitialize false
	} else {
		set ttinitialize true
	}



#		new_toplevel_tracked .tt "Create TrueType Text" prefs_window_geom_ttfont



		ac_frame $parent.f1
		ac_label $parent.f1.l -text "Font path"
		ac_entry $parent.f1.fontname -textvariable fontdir -width 0
		ac_button $parent.f1.browse -text "Browse..." -command { set fontdirtest [ttfont_get_fontdir $fontdir]; if { $fontdirtest != "" } {set fontdir $fontdirtest ; ttfont_fill_font_list $fontlistwidget $fontdir} }

		bind $parent.f1.fontname <KeyRelease> {
			ttfont_fill_font_list $fontlistwidget $fontdir
		}

		pack $parent.f1.l -side left
		pack $parent.f1.browse -side right -padx 3
		pack $parent.f1.fontname -fill x -expand true

		pack $parent.f1 -side top -expand 0

		# list of files in the font dir
		set f [ac_frame $parent.listframe]

		set fontlistwidget [listbox $f.list -yscrollcommand "$f.yscroll set" -height 8 -width 0 -selectmode single]

#		add_balloon $fontlistwidget "Available fonts"

		bind $fontlistwidget <ButtonRelease> {
			set fontname [selection get]
		}

		ac_scrollbar $f.yscroll -orient vertical -command " $f.list yview "

		pack $f.yscroll -side right -fill y -expand 0
		pack $f.list -side left -fill both -expand 1
		pack $f -fill both -expand 1 -padx 3 -pady 3


		# fill the font list
		ttfont_fill_font_list $fontlistwidget $fontdir


		ac_frame $parent.f2
		ac_frame $parent.f2.a
		ac_frame $parent.f2.b

		ac_label $parent.f2.a.fontl -text "Font"
		ac_entry $parent.f2.a.currentfontname -textvariable fontname
		pack $parent.f2.a.fontl -side left
		pack $parent.f2.a.currentfontname -fill x

		ac_label $parent.f2.b.textl -text "Text"
		ac_entry $parent.f2.b.text -textvariable fonttext
		bind $parent.f2.b.text <Return> " $parent.f9.go invoke "

		add_balloon $parent.f2.a.currentfontname "Selected font"
		add_balloon $parent.f2.b.text "Text to render"

		pack $parent.f2.b.textl -side left 
		pack $parent.f2.b.text -side left -fill x -expand true
		pack $parent.f2.a -side top -fill x
		pack $parent.f2.b -side top -fill x
		pack $parent.f2 -side top -pady 3

		ac_frame $parent.f3

		ac_labelframe $parent.f3.c -text "Object type"
		ac_radiobutton $parent.f3.c.objectoutline -text Outline -value "Outline" -variable objecttype -command " active_options_toggle $parent.f3.a"
		ac_radiobutton $parent.f3.c.objectsolid -text Solid -value "Solid" -variable objecttype -command " active_options_toggle $parent.f3.a"
		ac_radiobutton $parent.f3.c.objectextrude -text Extruded -value "Extrude" -variable objecttype -command " active_options_toggle $parent.f3.a"
		ac_radiobutton $parent.f3.c.objectbevel -text Beveled -value "Bevel" -variable objecttype -command " active_options_toggle $parent.f3.a"

		add_balloon $parent.f3.c "Choose object type to render"

		ac_labelframe $parent.f3.a -text "Parameters"
		ac_label $parent.f3.a.pointsizel -text "Font size"
		ac_entry $parent.f3.a.pointsize -textvariable pointsize -width 5
		ac_label $parent.f3.a.extrudedepthl -text "Extrude depth"
		ac_entry $parent.f3.a.extrudedepth -textvariable extrudedepth -width 5 -state disabled
		ac_label $parent.f3.a.beveldepthl -text "Bevel depth"
		ac_entry $parent.f3.a.beveldepth -textvariable beveldepth -width 5 -state disabled
		ac_label $parent.f3.a.bevelpct -text "%"
		ac_label $parent.f3.a.bevelanglel -text "Bevel angle"
		ac_spinbox $parent.f3.a.bevelangle -textvariable bevelangle -from "-90" -to 90 -increment 5 -width 5 -state disabled
		ac_label $parent.f3.a.detaill -text "Font detail"
		ac_spinbox $parent.f3.a.detail -textvariable fontdetail -from 1 -to 10 -increment 1 -width 5 -validate all -validatecommand { if { "%P"=="" } { return 1 }; catch { if { %P < 1 } { return 0 } }; return 1 }

		add_balloon $parent.f3.a.pointsize "Font point size"
		add_balloon $parent.f3.a.extrudedepth "Extrude depth"
		add_balloon $parent.f3.a.beveldepth "Bevel depth as percentage of extrude depth"
		add_balloon $parent.f3.a.bevelangle "Bevel angle"
		add_balloon $parent.f3.a.detail "Detail level for font"

		pack $parent.f3.c.objectoutline $parent.f3.c.objectsolid $parent.f3.c.objectextrude $parent.f3.c.objectbevel -side top -anchor w
		pack $parent.f3.c -side left -padx 3 -pady 3 -anchor nw

		grid $parent.f3.a.pointsizel $parent.f3.a.pointsize -sticky w
		grid $parent.f3.a.extrudedepthl $parent.f3.a.extrudedepth -sticky w
		grid $parent.f3.a.beveldepthl $parent.f3.a.beveldepth $parent.f3.a.bevelpct -sticky w
		grid $parent.f3.a.bevelanglel $parent.f3.a.bevelangle - -sticky w
		grid $parent.f3.a.detaill $parent.f3.a.detail - -sticky sw

		pack $parent.f3.a -side right -padx 3 -pady 3 -ipadx 2 -ipady 4 -anchor nw

		ac_frame $parent.f9
		ac_button $parent.f9.go -text "Create" -command ttfont_launch_create

		pack $parent.f9.go  -side left -padx 3

		pack $parent.f1 -side top -fill x -expand 0
		pack $parent.f2 -side top -fill x -expand 0
		pack $parent.f3 -side top -fill x -expand 0
		pack $parent.f9 -side top -pady 3 -expand 0


		# Initialize some dialog defaults
		if { $ttinitialize == true } {
			set pointsize 1
			set extrudedepth 0.1
			set beveldepth 40
			set bevelangle 45
			set fontdetail 5

			$parent.f3.c.objectsolid invoke
		}

		active_options_toggle $parent.f3.a

		
	# set focus on text entry widget:
	focus $parent.f2.b.text
}



proc ttfont_dialog {} {

	ac_toplevel_with_close .font3d "Add TrueType Text" ttfont_create_win
}



proc active_options_toggle { w } {

	global objecttype

	#set w .tt.f3.a

	switch $objecttype {
		"Outline" -
		"Solid" { $w.extrudedepth configure -state disabled; $w.beveldepth configure -state disabled; $w.bevelangle configure -state disabled }
		"Extrude" { $w.extrudedepth configure -state normal; $w.beveldepth configure -state disabled; $w.bevelangle configure -state disabled }
		"Bevel" { $w.extrudedepth configure -state normal; $w.beveldepth configure -state normal; $w.bevelangle configure -state normal }
	}
}
